/*	dmc_entry.c	02feb89 mwiederspahn utig
 *
 * modifed 07jul89 mw	if open for a+ fails, open for r
 *
 * This package permits random access to interim DMC holdings files.
 *
 * Higher level access to holdings entries is found in "dmc_holding.c"
 * which knows the assumptions built into the sort order of the file.
 * (This module has never been written, nor an interface designed.)
 *
 * This package assumes only:
 * a) read/write disk file, b) fixed length records
 * c) format strings and structures in "DMC.h" apply.
 *
 * This package is BSD UNIX specific (due to include files).
 */

/*
 * open_DMC_entry( name, file, size )	open a holdings file
 * read_DMC_entry( file, rec, buf )		read a record into buf
 * get_DMC_entry( file, rec, entry )	read a record into struct entry
 * write_DMC_entry( file, rec, buf )	write a record from buf
 * put_DMC_entry( file, rec, entry )	write a record from struct entry
 * close_DMC_entry( file )				close a holdings file
 */

#include <stdio.h>
#include <string.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "DMC.h"


int open_DMC_entry( name, file, size )
char *name;		/* input file name, will be created if needed */
FILE **file;	/* output file number */
int *size;		/* size in records */
{
struct stat statbuf;

if( (*file = fopen( name, "a+")) == NULL )
	if( (*file = fopen( name, "r")) == NULL )
		return( -2 );

	if( fstat( fileno(*file) , &statbuf ) == 0 )
		{
		*size = (int)statbuf.st_size / DMC_entry_reclen;
		return( 0 );
		}
	return( -1 );
}


int read_DMC_entry( file, rec, buf )
FILE *file;			/* open file */
int rec;			/* record number, 0=next */
char *buf;			/* output string [DMC_reclen] */
{
int status;


if( rec != 0 )
	if( (status=fseek( file, (rec-1)*DMC_entry_reclen,0 )) < 0)
		return( status );

return( fread( buf, sizeof(char), DMC_entry_reclen, file ) );
}


int get_DMC_entry( file, rec, entry )
FILE *file;				/* open file */
int rec;			/* record number, 0=next */
struct DMC_entry *entry;	/* returned entry */
{
char buf[DMC_entry_reclen+1];
register struct DMC_entry *eptr;
int status;
char	*p;

eptr = entry;
if( (status=read_DMC_entry( file, rec, buf )) == DMC_entry_reclen )
	{
	buf[DMC_entry_reclen] = (char)0;
	if( (status = sscanf( buf, DMC_entry_scan_format,
		eptr->statn, eptr->network, eptr->chn,
		&eptr->location, eptr->start, eptr->end,
		eptr->file, eptr->superfile)) == DMC_entry_count)
	{
		if( (p = strchr( eptr->file, ' ' )) != NULL ) *p='\0';
		if( (p = strchr( eptr->superfile, ' ' )) != NULL ) *p='\0';

		/* make sure the station name and channel and network are 
		 * padded with spaces to field length
		 */
		if (strlen(eptr->statn) < 5) 
		{
			strncpy(&eptr->statn[strlen(eptr->statn)], 
					"     ", 
					5 - strlen(eptr->statn));
		}

		if (strlen(eptr->chn) < 3)
                {
			strncpy(&eptr->chn[strlen(eptr->chn)], 
					"     ", 
					3 - strlen(eptr->chn));
                }
 
		if (strlen(eptr->network) < 2)
                {
			strncpy(&eptr->network[strlen(eptr->network)], 
				"     ", 
				2 - strlen(eptr->network));
                }

		if (strlen(eptr->location) < 2)
                {
                        strncpy(&eptr->location[strlen(eptr->location)],
                                "     ", 
                                2 - strlen(eptr->location));  
                }

		return(0);
	}
	else
		return( -1 );
	}
else
	return( -1 );
}
