/*===========================================================================*/
/* DMC interim out |            get_SEED_version           |         utility */
/*===========================================================================*/
/*
	Name:		get_SEED_version
	Purpose:	recover SEED_VERSION value from environment
	Usage:		float get_SEED_version ();
				float result;
				result = get_SEED_version ();
	Input:		none
	Output:		result = numeric equivalent of environment variable SEED_VERSION
	Externals:	none
	Messages:	none
	Warnings:	none
	Errors:		none
	Fatals:		none
	Called by:	anything
	Calls to:	none
	Algorithm:	read the value of the environment variable SEED_VERSION
	Notes:		usually called from main routine
				"result" should generally be available globally
	Problems:	none known
	Debug:		not used
	References:	none
	Language:	C
	Revisions:	written 19nov90, mark wiederspahn, from get_debug_level
*/

#include "output.h"
#include <sys/param.h>
#include <unistd.h>

int get_SEED_version ()
{
	char *charptr;							/* ptr to value of SEED_VERSION */
	float result;							/* numeric value of SEED_VERSION */

	char fname[MAXPATHLEN];
	char seed_v[20];	/* twenty ought to do it */

	if ((charptr = (char *) getenv ("SEED_VERSION")) != NULL)
		result = atof (charptr);
	else
		result = atof( SEED_DEFAULT );		/* internal default version */

	/* grab seed version from file...if there */
	sprintf(fname, "%s/SEED_VERSION", Header_path);

	if (access(fname, F_OK) == 0)
	{
		FILE *fptr;

		if ((fptr = fopen(fname, "r")) == (FILE *)NULL)	
		{
			fprintf(stderr, "WARNING! get_SEED_version: unable to open seed version file! Using default\n");
			perror("pod:get_SEED_version:");

			return ((int)(result*10.0 + 0.5));	
		
		}

		if (fgets(seed_v, sizeof(seed_v), fptr) == (char *)NULL)
		{
			fprintf(stderr, "WARNING! get_SEED_version: unable to read seed version file! Using default\n"); 
            perror("pod:get_SEED_version:"); 

			fclose(fptr);
 
            return ((int)(result*10.0 + 0.5)); 
		}  	

		result = atof(seed_v);

		fclose(fptr);

	}		/* if file is there */


	if( Debug )
		fprintf( D_OUT,"[get_SEED_version] we will write version %f\n",result );

/*
 * round to nearest integer of *10 value
 */
	return ( (int)(result*10.0 + 0.5) );
}
