/*===========================================================================*/
/* UTIG DMC        |            make_lvol.c               |  Utility         */
/*===========================================================================*/
/*
	Name:		make_lvol.c

	Purpose:	allocate and link an lvol node
	Usage:		struct lvol *make_lvol( head, tail )
	Input:		struct	lvol	**head
				struct	lvol	**tail
	Output:		pointer to created node
	Externals:	
	Warnings:	
	Errors:		
	Fatals:		calloc returns no new space
	Called by:	
	Calls to:	error_handler()
	Algorithm:	If the list is empty, then both head and tail will be null.
				(eg *head = NULL). If there are any nodes on the list, the
				*head and *tail point to the ends; a node is linked in
				in either case.
	Notes:		
	Problems:	
	References:	
	Language:	
	Revisions:	03/10/89	mark wiederspahn	written
*/

#include "output.h"

struct lvol *make_lvol( head, tail )
struct	lvol	**head;
struct	lvol	**tail;
{
/*
 * if this is the first node, point both head and tail to new node
 * else add one onto the existing list at the end.
 */
	if( *tail == NULL )
	{
		*head = *tail =
			(struct lvol *)calloc( 1, sizeof(struct lvol) );
		if( *tail == NULL ) error_handler( FATAL,"[make_lvol] no space");
	}
	else
	{
		(*tail)->next = 
			(struct lvol *)calloc( 1, sizeof(struct lvol) );
		if( *tail == NULL ) error_handler( FATAL,"[make_lvol] no space");
		(*tail) = (*tail)->next;
	}

	return( *tail );
}
