#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h> 

#include "rdseed.h"

#define SUBSTR(str,xch,ch)      \
        { char *c; \
          while (c = strchr(str, xch)) \
                *c = ch; \
        }

#define IS_BTW(x,y,z) \
        ((timecmp(x, y) >= 0) && (timecmp(x, z) <= 0))

 
#define append_linklist_element(new, head, tail) \
    new->next = NULL; \
    if (head != NULL) tail->next = new; \
    else head = new; \
    tail = new;

struct twindow 
{
	char time_start[23]; 
        char time_end[23];        
	int already_used;

	char chn_list[512];

	struct twindow *next;
};


struct stn_tspan
{
	char net[5];
	char stn[10];

	char *this_event;
	
	struct twindow *ts_head;
	struct twindow *ts_tail;

	struct stn_tspan *next;	

} *stn_listhead, *stn_listtail;

static char *this_event = "";

char *add_time();
void dump_stn_nodes();
void dump_twindows();

extern float WeedVersion;
extern int output;


read_summary_file(fname)
char *fname;

{
	FILE *sumfile;

	char buffer[512];
	char *this_ev, *ch;
	char token[30];
	int  bias_start;
	int  bias_end;

	/* ### warning  Must be big enough for indeterminate channel count */
	char chn[3000]; 


	char time_win_start[23];
	char time_win_end[23];
	
	struct stn_tspan *stn_ptr;
	struct twindow *twindow_ptr;

	sumfile = fopen(fname, "r");

	if (sumfile == NULL)
	{
		fprintf(stderr, "Unable to open summary file: %s!\n", fname);
		perror("read_summary_file");
		return(0);
	}

	/* read in the first token */
	while (fscanf(sumfile, "%s", token) != EOF)
	{
		/* scan first tokens, EVENT, STATION, or PHASE */
		if (strcmp(token, "") == 0)
			continue;

		if (strstr(token, "EVENT"))
		{
			/* save Event line */
			fgets(buffer, 512, sumfile);

			/* erase \n at end */
			buffer[strlen(buffer) - 1] = 0;

			this_ev = malloc(strlen(buffer));

			if (this_ev == NULL)
			{
				fprintf(stderr, "Error : unable to obtain memory for event from summary file!\n");
				fprintf(stderr, "Quitting\n");

				exit(-1);

			}

			/* save it for later */
			strcpy(this_ev, buffer);
	
			continue;

		}
		if (strstr(token, "STATION")) 
                {
			stn_ptr = (struct stn_tspan *)
					malloc(sizeof(struct stn_tspan));

			        
                        if (stn_ptr == NULL)   
                        {
                                fprintf(stderr, "Error : unable to obtain memory for station from summary file!\n"); 
                                fprintf(stderr, "Quitting\n");
 
                                exit(-1);
 
                        }
        
			memset((char *)stn_ptr, 0, sizeof(struct stn_tspan));

			fscanf(sumfile, 
				"%s %s", 
				stn_ptr->stn, stn_ptr->net);

			stn_ptr->this_event = this_ev;

			append_linklist_element(stn_ptr, 
						stn_listhead, stn_listtail);

			add_stn(stn_ptr->stn);

			fgets(buffer, 512, sumfile);

			continue;
                }

		if (strstr(token, "PHASE")) 
                {
			twindow_ptr = (struct twindow *)
                                        malloc(sizeof(struct twindow));

			memset((char *)twindow_ptr, 0, sizeof(struct twindow));

			fscanf(sumfile, "%*s %d %*s %d %s %s %s %*s\n",
					&bias_start,
					&bias_end,
					chn,
					time_win_start,
					time_win_end);

			/* strip out the quotes in the channel list */
			strncpy(twindow_ptr->chn_list, 
				&chn[1], /* start quote */
				strlen(chn) > 511 ? 511 : strlen(chn) - 2);

			/* now the ending quote */
			twindow_ptr->chn_list[strlen(chn) > 511 ? 
						511 : strlen(chn) - 2] = 0;

			add_chn(twindow_ptr->chn_list);

			/* WEED version -1.5 and greater allows the user
			 * to enter negative or positive values here.
			 * If < 2.0 assume subtraction only, make into 
			 * negitive
			 */
			strcpy(twindow_ptr->time_start, 
				add_time(time_win_start, 
					  WeedVersion >= 1.5 ? 
							bias_start :
							-bias_start));

			
			strcpy(twindow_ptr->time_end, 
				add_time(time_win_end, bias_end));

			append_linklist_element(twindow_ptr,
                                                stn_ptr->ts_head, 
						stn_ptr->ts_tail);

	
			setup_tspan(twindow_ptr->time_start,
					twindow_ptr->time_end);

			continue;
                }
		
	}		/* while fscanf */

	fclose(sumfile);

	/* must parse channel_list as built in add_chn(), parse
	 * into channel_point, all globals, used by parsers()
	 */
	channel_count = 0;

	ch = strtok(channel_list, ",");

	if (ch != NULL)	
		do 
		{
			channel_point[channel_count] = ch;
			channel_count++;

		} while ((ch = strtok(NULL, ",")) != NULL);

	return 1;
	
}

/* ------------------------------------------------------------------- */

char *strptime();

char *ddd2mmdd();


/* --------------------------------------------------------------------- */
void dump_stn_nodes(stn_node)
struct stn_tspan *stn_node;

{
	while (stn_node)
	{
		printf("%s / %s:%p\n", stn_node->net, stn_node->stn, stn_node);

		dump_twindows(stn_node->ts_head);

		stn_node = stn_node->next;
	}

}

/* -------------------------------------------------------------------- */
int add_stn(stn)
char *stn;

{
	int i;
	int found = FALSE;

	for (i = 0; i < station_count; i++)
		if (strcmp(station_point[i], stn) == 0)
			found = TRUE;

	if (!found)
	{
		station_point[i] = stn;
		station_count++;

	}

	return;

}

/* -------------------------------------------------------------------- */
int add_chn(chn)
char *chn;

{
	int i;
        int found = FALSE;
	char *ch;

	char *ch_ptr;
	char channels[512];

	/* protect original string - also channel_list is global */
	strncpy(channels, chn, strlen(chn));
	channels[strlen(chn)] = 0;

	/* the time window channel string could be comma delimited */
	ch = strtok(channels, ",");

	while(ch != NULL)
	{
        	if (strstr(channel_list, ch) == 0)
        	{

			strcat(channel_list, ch);
			strcat(channel_list, ",");
        	}

		ch = strtok(NULL, ",");
	}

        return;

}

/* --------------------------------------------------------------------- */
void dump_twindows(twin)
struct twindow *twin;

{
        char time_start[23];
        char time_end[23];

        char chn_list[100];

	while (twin)
	{

		printf("\t%s <> %s - %s\n", 
				twin->time_start, twin->time_end,
				twin->chn_list);

		twin = twin->next;
	}

	return;
}

/* ------------------------------------------------------------------------- */
int free_stn_nodes()
{
	free_stn(stn_listhead);	

	stn_listhead = NULL;
	stn_listtail = NULL;

	return;
}

/* ------------------------------------------------------------------------- */
int free_stn(node)
struct stn_tspan *node;

{
	if (node == NULL)
	{
		return;
	}
	else
	{
		free_stn(node->next);

		free_tspans(node->ts_head);

		node->ts_head = NULL;
		node->ts_tail = NULL;
		
		free(node->this_event);

		free(node);

		node = NULL;

	}

	return;


}

/* ------------------------------------------------------------------------- */
int free_tspans(ts_node)
struct twindow *ts_node;

{
        if (ts_node == NULL)
        {
                return;
        }
        else
        {   
                free_tspans(ts_node->next);

                free(ts_node);

		ts_node = NULL;
        }

        return;


}

/* ------------------------------------------------------------------------- */
	
extern long timezone;
extern int daylight;


char *add_time(time_asc, incr)
char *time_asc;		/* YYYY,DDD,HH:MM:SS.FFFF */
int incr;

{
	time_t t;
	struct time tm, newtime;

	char YYYYDDDetc[30];

	memset((char *)&tm, 0, sizeof(struct tm));

	SUBSTR(time_asc, ',', ' ');

	SUBSTR(time_asc, ':', ' ');

	SUBSTR(time_asc, '.', ' ');

	sscanf(time_asc, "%d %d %d %d %d", 
				&tm.year, &tm.day,
				&tm.hour, &tm.minute, &tm.second,
				&tm.fracsec);

	if (incr < 0)
		newtime = timesub(tm, (-incr) * 10000);
	else
		newtime = timeadd(tm, incr * 10000);

	sprintf(YYYYDDDetc, "%d,%03d,%02d:%02d:%02d.%d", 
		newtime.year, 
		newtime.day,
		newtime.hour, newtime.minute, 
		newtime.second, newtime.fracsec);

	return YYYYDDDetc;		
}

/* ---------------------------------------------------------------------- */

#define isaleap(year) ((((year)%100 != 0) && ((year)%4 == 0)) || ((year)%400 == 0))


/* ----------------------------------------------------------------------- */
static int days_in_month[] = {0, 31, 28, 31, 30, 31, 30, 
				31, 31, 30, 31, 30, 31};

char *ddd2mmdd(ddd, yyyy)
int ddd;
int yyyy;

{

	static char mmddyyyy[200];
	
	int mon;

	for (mon = 1; mon < 13; mon++)
	{
		if (ddd <= days_in_month[mon])
			break;

		ddd -= days_in_month[mon];

		/* adjust for leap year */

		if (isaleap(yyyy) && (mon == 2))
			ddd--;

	}

	if (mon == 13)	
	{
		fprintf(stderr, "Bad day number!\n");
		return;
	}


	sprintf(mmddyyyy, "%d/%d/%d", mon, ddd, yyyy);	

	return mmddyyyy;
}

/* ------------------------------------------------------------------------- */
struct twindow *scan_summary_file(stn, chn, net, blk_start, blk_end)
char *stn, *chn, *net;
struct time *blk_start;
struct time *blk_end;

{
        char **c_ptr;
	struct stn_tspan *stn_ptr;

	struct time req_start;
	struct time req_end;

	int found_chn;
	
        int i, n;

        stn_ptr = stn_listhead;
 
        while (stn_ptr)
        {
		found_chn = FALSE;

               	if ((strcmp(stn_ptr->stn, stn) != 0)) 
		{
			stn_ptr = stn_ptr->next;
			continue;
		}

		if (strcmp(stn_ptr->net, "??") == 0 ? 0 : 
                        (strcmp(stn_ptr->net, net) != 0))
		{
			stn_ptr = stn_ptr->next;
                        continue;
		}


		/* ckeck phase lines, comparing channel */

		strupr(stn_ptr->ts_head->chn_list);
 
		n = split(stn_ptr->ts_head->chn_list, &c_ptr, ",");
 
                for (i = 0; i < n; i++)
		{
			if (wstrcmp(chn, c_ptr[i], 3) == 0)
				found_chn = TRUE;
	
		}

		fuse(&c_ptr, n);

		if (!found_chn)
		{
			stn_ptr = stn_ptr->next;
			continue;
		}		

               	timecvt(&req_start, stn_ptr->ts_head->time_start);
                timecvt(&req_end, stn_ptr->ts_head->time_end);

		/* at last - compare times */
		
		if (!((IS_BTW(req_start, *blk_start, *blk_end))   ||
			(IS_BTW(req_end, *blk_start, *blk_end))  ||
			(IS_BTW(*blk_start, req_start, req_end))  ||
			(IS_BTW(*blk_end, req_start, req_end))))
                {
			stn_ptr = stn_ptr->next;
                        continue;
                }
	
		/* got here - everything checks out */

		return stn_ptr->ts_head;	
	}

	return 0;	/* didn't find it */
	
}

/* ------------------------------------------------------------------------- */

#if 0

struct twindow *scan_summary_file(stn, chn, net, start, end, tag_it_used)
char *stn, *chn, *net, *start, *end;
int tag_it_used;

{
	struct stn_tspan *stn_ptr = stn_listhead;
	struct twindow *twin, *tw;

	char *ch_ptr;

	char now_chn[1024];
	int ret;

	twin = NULL;
	tw = NULL;

	trim(net);

	/* cycle thru list until a matching stn/net combo is found */
	while (stn_ptr)
	{

		if ((strcmp(stn_ptr->stn, stn) == 0) &&
			(strcmp(stn_ptr->net, "??") == 0 ? 1 :
			(strcmp(stn_ptr->net, net) == 0)))
		{
			/* cycle thru the phase lines, comparing channel */
			twin = stn_ptr->ts_head;

			while (twin)
			{

				strupr(twin->chn_list);

				strcpy(now_chn, twin->chn_list);
				ch_ptr = strtok(now_chn, ",");
				if (ch_ptr == NULL)
				 	ch_ptr = "";  /* default all channels*/  
				while(ch_ptr != NULL)
				{
				if ((wstrcmp(chn, ch_ptr, 3) == 0) ||
					(strcmp(ch_ptr, "") == 0)  &&
					(!twin->already_used))
					    if (ret = chk_tspans(start, end,
							twin->time_start,
							twin->time_end))
					    {
						/* bingo */
						this_event = stn_ptr->this_event;

						twin->already_used = tag_it_used;
						tw = twin;

						if (ret == 4)
							return tw;

					    }

					ch_ptr = strtok(NULL, ",");

				}

				twin = twin->next;
			}

		}

		stn_ptr = stn_ptr->next;
	}

	return tw;

}
#endif

/* ---------------------------------------------------------------------- */
char *get_channel_name();
int reset_chn_name_generator();

struct type74 *extract_74s();

static struct stn_tspan *stn_ptr;

int process_summary_file()

{
        struct twindow *twin, *tw;
	
	struct type74 *b74_ptr;
 
	char *chn_ptr, **c_ptr;
	char channel[4];

	int i, n;

	stn_ptr = stn_listhead;

	while (stn_ptr)	
	{

		/* 1st check to see if stn, net is in the contents */
		if (!scan_74s(stn_ptr->stn, stn_ptr->net))
		{
			stn_ptr = stn_ptr->next;
			continue;
		}

		n = split(stn_ptr->ts_head->chn_list, &c_ptr, ",");

		for (i = 0; i < n; i++)
		{
			if (strchr(c_ptr[i], '*') || strchr(c_ptr[i], '*'))
			{
				reset_chn_name_generator(c_ptr[i]);

				chn_ptr = get_channel_name();
			}	
			else
				chn_ptr = c_ptr[i];
			

			if (chn_ptr != NULL)
				strcpy(channel, chn_ptr);

			while (chn_ptr != NULL)
			{

				/* Notice: only one twindow per stn node, no linked
		 	 	 * list for now, each time window gets a separate node
		 	  	 */

				b74_ptr = extract_74s(stn_ptr->stn,
						      stn_ptr->net,
					   		channel,
					   		stn_ptr->ts_head->time_start, 
					   		stn_ptr->ts_head->time_end);

				if (b74_ptr != NULL)
				{
					this_event = stn_ptr->this_event;

					extract_this_timespan(b74_ptr);

					time_span_out();


					output = 0;

				}

				if (strchr(c_ptr[i], '*') || strchr(c_ptr[i], '*'))    
					chn_ptr = get_channel_name(); 
				else
					chn_ptr = NULL;

				if (chn_ptr != NULL)
					strcpy(channel, chn_ptr);

			}  /* while chn_ptr */

		}		/* for */

		fuse(&c_ptr, n);

		stn_ptr = stn_ptr->next;
	}

}

/* ------------------------------------------------------------------------- */
struct twindow *get_twin_from_summary()

{
	return stn_ptr->ts_head;
}

/* ------------------------------------------------------------------------- */

char *band	= "ESHBMLVURAWX";
char *inst 	= "HLGM";
char *orient 	= "ZNEABCTR123UVW";

static int now_band = 0, now_inst = 0, now_orient = 0;
static int which_chn_code;

char *get_channel_name()

{
	static char chn[5];

	/* at end of list, need to reset */
	if (now_band >= strlen(band))
		return NULL;


	if (now_orient >= strlen(orient))
	{
		if (which_chn_code == 2)
		{
			/* don't do carry over, stop at the end of 
			 * orientation char list 
			 */
			return NULL;
		
		}

		now_orient = 0;
		now_inst++;

		if (now_inst >= strlen(inst))
		{
			/* Same with this code */
			if (which_chn_code == 1)
				return 0;

			now_inst = 0;
			now_band++;

			if (now_band >= strlen(band))
				return NULL;
		}
	}

	chn[0] = band[now_band];
	chn[1] = inst[now_inst];
	chn[2] = orient[now_orient];

	chn[3] = 0;

	now_orient++;

	return chn;

}

int reset_chn_name_generator(seed)
char *seed;

{

	if (strlen(seed) == 0)
	{
		which_chn_code = -1;
		now_band = now_inst = now_orient = 0;

		return 0;
	}

	if ((strchr(seed, '*') == 0) && 
		(strchr(seed, '?') == 0))
	{
                now_band = now_inst = now_orient = 0;
		which_chn_code = -1;
                return 0; 
	}

	
	if ((seed[0] != '*') && (seed[0] != '?'))
	{
		now_band = strchr(band, seed[0]) - band;
	}
	else
	{
		now_band = 0;
		which_chn_code = 0;
	}

        if ((seed[1] != '*') && (seed[1] != '?'))
        {
		now_inst = strchr(inst, seed[1]) - inst;
        } 
	else
	{
		now_inst = 0;
		which_chn_code = 1;
	}
 
        if ((seed[2] != '*') && (seed[2] != '?'))
        {
               	now_orient = strchr(orient, seed[2]) - orient;
        } 
	else
	{
		now_orient = 0;
		which_chn_code = 2;
	}

	return 0;

}

/* ------------------------------------------------------------------------- */

struct type74 *find_closest_b74();

/* ----------------------------------------------------------------- */
struct type74 *extract_74s(stn, net, chn, start, end)
char *stn, *net, *chn, *start, *end;

{

/*
 *	We are looking at these possibilities
 *
 *	Request from summary file totally lies within b74 start, end
 *		stop here as we found it
 *
 *	Search again for a B74 that overlaps it, or a B74 which lies
 *	totally within the requested timespan. Determine which is
 *      closer.
 */
	
	struct type74 *type74;
	int found_times, found_stn;

	type74 = type74_head;

	found_times = found_stn = FALSE;

	while (type74 != NULL)
	{
                if (strcmp(stn, type74->station) != 0)
		{
			type74 = type74->next;
			continue;
		}

		if (strcmp(net, "??") == 0 ? 0 :
                        (strcmp(net, type74->network_code) != 0))
                {

			type74 = type74->next;
                        continue;

		}


		if (chk_chn_list(chn, type74->channel) == 0)
                {
 
                        type74 = type74->next;
                        continue;
                }
         

		found_stn = TRUE;

		/* check B74's start end time pairs which will completely 
		 * encompass the request start/end times 
		 */
		if (chk_times(start, end, type74->starttime, type74->endtime))
		{
			found_times = TRUE;

			break;
		}
	
                type74 = type74->next;
        }

	/* if encompassing 74 not found, look for best fit */

	if (!found_times && found_stn)
	{
		type74 = find_closest_b74(net, 
					  stn, 
					  chn, 
					  start, 
					  end, type74_head);

		if (type74 == NULL)
		{
#if 0
			fprintf(stderr, "Warning : scan_74s() - unable to locate matching blockette 74 for summary file requested entry\n");

			fprintf(stderr, "Request: station/chn/net %s/%s/%s for :%s..to..%s\n", 
						stn, 
						chn, 
						net, 
						start, 
						end);
#endif

		}
	}

	return type74;
	
}
/* ----------------------------------------------------------------- */

struct type74 *find_closest_b74(net, stn, chn_list, start, end, type74)
char *net;
char *stn;
char *chn_list;
char *start, *end;
struct type74 *type74;

{

	struct type74 *save_type74 = NULL;

	struct time req_start, tspan_start;
	struct time req_end, tspan_end;

	float diff_start, diff_end, this_diff;

	float smallest = 999999999;

	while (type74 != NULL)
	{
		if (strcmp(stn, type74->station) != 0)
		{
			type74 = type74->next;
			continue;
		}

		if (strcmp(net, "??") == 0 ? 0 :
			(strcmp(net, type74->network_code) != 0))
		{

			type74 = type74->next;
			continue;

		}

		if (chk_chn_list(chn_list, type74->channel) == 0)
		{
			type74 = type74->next;
			continue;
		}
 
 
		/* check B74's start end time pairs which
		 * come closest to the request start/end times
                 */

		/* strategy is to add up the differences of the
		 * corresponding endpoints, smallest wins
		 */

		/* 1st check for overlap */

		timecvt(&req_start, start);
		timecvt(&tspan_start, type74->starttime);

	
		timecvt(&req_end, end);
                timecvt(&tspan_end, type74->endtime);

		if (!((IS_BTW(req_start, tspan_start, tspan_end))   ||
			(IS_BTW(req_end, tspan_start, tspan_end))  ||
			(IS_BTW(tspan_start, req_start, req_end))  || 
                        (IS_BTW(tspan_end, req_start, req_end))))
		{
			type74 = type74->next;
			continue;
		}
		     
		diff_start = timedif(req_start, tspan_start);
                diff_end = timedif(req_end, tspan_end);
                
		this_diff = fabs(diff_start) + fabs(diff_end);
 
                if (this_diff < smallest)
		{
			smallest = this_diff;

			save_type74 = type74;

		}

		type74 = type74->next;
 
	}

	return save_type74;

}

/* ----------------------------------------------------------------- */
int scan_74s(stn, net)
char *stn, *net;

{
        struct type74 *type74;

        type74 = type74_head;

        while (type74 != NULL)
        {
                if (strcmp(stn, type74->station) != 0)
                {
                        type74 = type74->next;
                        continue;
                }

                if (strcmp(net, "??") == 0 ? 0 :
                        (strcmp(net, type74->network_code) != 0))
                {

                        type74 = type74->next;
                        continue;
 
                }

		return 1;
 
	} 	

	return 0;

}

/* ----------------------------------------------------------------- */

int chk_times(r_start, r_end, B74_start, B74_end)
char *r_start, *r_end, *B74_start, *B74_end;

{
        struct time b74_start, b74_end;
        struct time req_start, req_end;

        timecvt(&b74_start, B74_start);
        timecvt(&b74_end, B74_end);

        timecvt(&req_start, r_start);
        timecvt(&req_end, r_end);


	/* check for overlap of the B74 over the requested start/stop */

	return((timecmp(b74_start, req_start) <= 0) && 
		(timecmp(b74_end, req_end) >= 0));
}

/* ----------------------------------------------------------------------- */
int chk_chn_list(chn_list, channel)
char *chn_list;
char *channel;

{
	char wrkstr[512];
	char *ch_ptr;

	strcpy(wrkstr, chn_list);

	strupr(wrkstr);
	strupr(channel);
	
	ch_ptr = strtok(wrkstr, ",");

	while (ch_ptr != NULL)
	{
		if (wstrcmp(channel, ch_ptr, 3) == 0)
			return 1;

		ch_ptr = strtok(NULL, ",");

	}

	return 0;
	
}

/* ----------------------------------------------------------------------- */

char *summary_file_get_event()
 
{
	return this_event;

}

/* ------------------------------------------------------------------------- */
#if 0 

int chk_tspans(start_1, end_1, start_2, end_2)
char *start_1, *end_1, *start_2, *end_2;

{
	struct time blk_start, blk_end;
	struct time req_start, req_end;

	timecvt(&blk_start, start_1);
	timecvt(&blk_end, end_1);
	
        timecvt(&req_start, start_2);
        timecvt(&req_end, end_2); 

	/* Look at 4 possibilities :
	 *
 	 * 1: starts are equal i.e:
	 *	blk |----------------------| 
	 *      req |---------------------------| 
	 *
	 * 2. start of blk is greater than req start, 
	 *       but not greater than req end, i.e:
	 *     blk       |------------------------------|
	 *     req  |-------------------------|
	 *
	 * 3. start of blk before req start, but end is before req end, i.e:
	 *      blk |--------------------------|
	 *      req              |-----------------|
	 * 
	 * 4. blk overlaps request
	 *     blk |--------------------------|
	 *     req      |-----------------|
	 * 	
	 */

	/* #4 */
	if ((timecmp(req_start, blk_start) >= 0) &&
		(timecmp(req_end, blk_end) <= 0))
		return 4;

	/* # 1 - does start of block == request ? */
	if (timecmp(blk_start, req_start) == 0)
		return 1;

	if (timecmp(blk_start, req_start) > 0)
	{
	/* #2 */
		if (timecmp(blk_start, req_end) <= 0)
			return 1;
	}

	/* # 3*/
	if ((timecmp(blk_end, req_start) > 0) &&
		(timecmp(blk_end, req_end) <= 0))
		return 1;

	/* Got here? didn't find it */
	return 0;
	
}
#endif

/* ------------------------------------------------------------------------- */
int setup_tspan(tspan_start, tspan_end)
char *tspan_start, *tspan_end;

{
	/* fake out computer into thinking this timespan was entered 
	 * at the computer. Add into holding array... */

	start_time_count++;
	end_time_count++;

	if (start_time_count == 1)	/* 1st time through */
		start_time_point = (struct time *)malloc(sizeof(struct time));
	else
                start_time_point = (struct time *)realloc(start_time_point,
							   sizeof(struct time) * start_time_count);

	if (end_time_count == 1)
		end_time_point = (struct time *)malloc(sizeof(struct time));
	else
		end_time_point = (struct time *)realloc(end_time_point,
							sizeof(struct time) * end_time_count);

	timecvt(&start_time_point[start_time_count - 1], tspan_start);
	timecvt(&end_time_point[end_time_count - 1], tspan_end);

}

/* ------------------------------------------------------------------------- */


